package org.churchsw.scripture.studyware;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.Position;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

/**
 * The Class EnglishBible.
 */
public class EnglishBible {

	/** The _ title. */
	static String _Title = "Scripture StudyWare";

	/** The bible_books. */
	JTree bible_books = null;
	JScrollPane chapter_jsp = null;
	JPanel chapterPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));

	int m_book_no = 1;
	int m_chapter_no = 1;

	/** The books. */
	Object[] books = null;

	/** The inc. */
	int inc = 0;

	/** The default font size. */
	int defaultFontSize = 10;

	LoadingDialog loading = null;

	/** The frame. */
	protected static JFrame frame = null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.plugin.UI.Windows.IWindow#getFrame()
	 */
	public JFrame getFrame() {
		return frame;
	}

	/**
	 * Instantiates a new english bible.
	 */
	public EnglishBible() {
		loading = new LoadingDialog(frame);
		doInit(1, 1, 1, false);		
		frame.setIconImage(new ImageIcon("bible.gif").getImage());
		frame.setVisible(true);
	}

	/**
	 * Instantiates a new english bible.
	 * 
	 * @param mbook
	 *            the mbook
	 * @param mchapter
	 *            the mchapter
	 * @param mverse
	 *            the mverse
	 */
	public EnglishBible(int mbook, int mchapter, int mverse) {
		doInit(mbook, mchapter, mverse, true);
	}

	/**
	 * Instantiates a new english bible.
	 * 
	 * @param mbook
	 *            the mbook
	 * @param mchapter
	 *            the mchapter
	 * @param mverse
	 *            the mverse
	 * @param external_call
	 *            the external_call
	 * @param bibleVersion
	 *            the bible version
	 * @param bibleVersionName
	 *            the bible version name
	 */
	public EnglishBible(int mbook, int mchapter, int mverse,
			boolean external_call) {
		doInit(mbook, mchapter, mverse, external_call);
	}

	/**
	 * Do init.
	 * 
	 * @param mbook
	 *            the mbook
	 * @param mchapter
	 *            the mchapter
	 * @param mverse
	 *            the mverse
	 * @param external_call
	 *            the external_call
	 * @param bibleVersion
	 *            the bible version
	 * @param bibleVersionName
	 *            the bible version name
	 */
	private void doInit(int mbook, int mchapter, int mverse,
			boolean external_call) {

		_Title = "StudyWare Bible "+StrongDB.version+" [KJV 1769]";
		frame = new JFrame(_Title);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JSplitPane jsp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		jsp.setDividerLocation(200);
		try {
			books = StrongDB.getBibleBooks();
			Object[] oldt_books = new Object[40];
			oldt_books[0] = "Old Testament";
			for (int i = 0; i < 39; i++) {
				int chap_count = StrongDB.getChapterCount(i + 1);
				String[] chapters = new String[chap_count + 1];
				chapters[0] = (i + 1) + ". " + (String) books[i];
				for (int j = 0; j < chap_count; j++)
					chapters[j + 1] = "Chapter " + Integer.toString(j + 1);
				oldt_books[i + 1] = chapters;
			}

			Object[] newt_books = new Object[28];
			newt_books[0] = "New Testament";
			for (int i = 39; i < 66; i++) {
				int chap_count = StrongDB.getChapterCount(i + 1);
				String[] chapters = new String[chap_count + 1];
				chapters[0] = (i + 1) + ". " + (String) books[i];
				for (int j = 0; j < chap_count; j++)
					chapters[j + 1] = "Chapter " + Integer.toString(j + 1);
				newt_books[i + 1 - 39] = chapters;
			}

			bible_books = new JTree(processHierarchy(new Object[] {
					"Holy Bible", oldt_books, newt_books }));
			bible_books.addTreeSelectionListener(new TreeSelectionListener() {

				public void valueChanged(TreeSelectionEvent e) {
					TreePath path = e.getPath();
					int book_no = -1;
					int chapter_no = -1;

					if (path.getPathCount() == 4) {
						chapter_no = Integer.parseInt(path.getPath()[3]
								.toString().split(" ")[1]);
						book_no = Integer.parseInt(path.getPath()[2].toString()
								.split("[.]")[0]);
						m_book_no = book_no;
						m_chapter_no = chapter_no;
						BackLoad bl = new BackLoad(book_no, chapter_no);
						bl.start();

					}
				}
			});
		} catch (Exception e) {
			e.printStackTrace();
		}
		JScrollPane books_jsp = new JScrollPane(bible_books);
		jsp.add(books_jsp, 0);

		JPanel top = new JPanel(new FlowLayout(FlowLayout.CENTER));

		JLabel pluginslbl = new JLabel("Plugins: ");
		top.add(pluginslbl);

		String pluginsList[] = StrongDB.plugins.getProperty("plugins").split(
				",");

		JComboBox pluginsCombo = new JComboBox(pluginsList);

		pluginsCombo.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				String classname = StrongDB.plugins
						.getProperty((String) ((JComboBox) e.getSource())
								.getSelectedItem());
				StrongDB.reloadAfterPluginExecute=false;
				try {
					Class.forName(classname).newInstance();
				} catch (InstantiationException e1) {
					JOptionPane.showMessageDialog(null, "Error:"
							+ e1.getLocalizedMessage());
					e1.printStackTrace();
				} catch (IllegalAccessException e1) {
					JOptionPane.showMessageDialog(null, "Error:"
							+ e1.getLocalizedMessage());
					e1.printStackTrace();
				} catch (ClassNotFoundException e1) {
					JOptionPane.showMessageDialog(null, "Error:"
							+ e1.getLocalizedMessage());
					e1.printStackTrace();
				}

				if (StrongDB.reloadAfterPluginExecute) {
					BackLoad bl = new BackLoad(m_book_no, m_chapter_no);
					bl.start();
				}
			}
		});
		top.add(pluginsCombo);

		JButton about=new JButton("About");
		about.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(frame, 
						"<html><h2>About StudyWare Bible "+StrongDB.version+"</h2>" +
						"Build-date: "+StrongDB.builddate+"\n" +
						"Website: www.churchsw.org\n\n" +
						"StudyWare Bible is a tool that displays the list of translated English words for a particular\n" +
						"Hebrew and Greek word in the scriptures. Hence, you may not need to know Greek or Hebrew, but\n" +
						"you can look into the other English words translated for the same word in original language.\n" +
						"This Bible is only for King James Version (1769).\n" +
						"\n" +
						"Warning!: Without Word of God in your heart who is Christ Himself, this tool or any scripture is a\n" +
						"garbage. You can have a thousand tools, and even know Greek and Hebrew, yet you learn nothing. If\n" +
						"you have His Word in you, the scripture is a treasure book for every verse testifies Christ.\n" +
						"\n" +
						"StudyWare Bible by Felix Jeyareuben (felix@fjeyar.com) through Jesus Christ's help. You can\n" +
						"download the software from the website.\n" +						
						"This software is free, opensource and released to public domain","About StudyWare Bible 1.0",JOptionPane.INFORMATION_MESSAGE);				
			}});
		top.add(about);
		
		JPanel content = new JPanel(new BorderLayout());

		content.add(top, BorderLayout.NORTH);
		chapter_jsp = new JScrollPane(chapterPanel);
		chapter_jsp.setPreferredSize(new Dimension(800, 600));
		content.add(chapter_jsp, BorderLayout.CENTER);
		jsp.add(content, 1);

		frame.getContentPane().add(jsp);
		frame.pack();

		String ntot = "";
		if (mbook < 40)
			ntot = "Old Testament";
		else
			ntot = "New Testament";
		String bookname = "";
		try {
			bookname = StrongDB.getBookName(mbook-1);
		} catch (Exception e1) {

		}
		bible_books.expandPath(bible_books.getNextMatch("Holy Bible", 0,
				Position.Bias.Forward));
		bible_books.expandPath(bible_books.getNextMatch(ntot, 0,
				Position.Bias.Forward));
		bible_books.expandPath(bible_books.getNextMatch(
				mbook + ". " + bookname, 0, Position.Bias.Forward));
		TreePath selectedPath = bible_books.getNextMatch("Chapter " + mchapter,
				0, Position.Bias.Forward);
		bible_books.expandPath(selectedPath);
		bible_books.setSelectionPath(selectedPath);

		frame.setLocationRelativeTo(null);
		
//		BackLoad bl = new BackLoad(m_book_no, m_chapter_no,true);
//		bl.start();
		
		frame.setVisible(true);
	}

	/**
	 * Process hierarchy.
	 * 
	 * @param hierarchy
	 *            the hierarchy
	 * 
	 * @return the default mutable tree node
	 */
	private DefaultMutableTreeNode processHierarchy(Object[] hierarchy) {
		DefaultMutableTreeNode node = new DefaultMutableTreeNode(hierarchy[0]);
		DefaultMutableTreeNode child;
		for (int i = 1; i < hierarchy.length; i++) {
			Object nodeSpecifier = hierarchy[i];
			if (nodeSpecifier instanceof Object[]) // Ie node with children
				child = processHierarchy((Object[]) nodeSpecifier);
			else
				child = new DefaultMutableTreeNode(nodeSpecifier); // Ie Leaf
			node.add(child);
		}
		return (node);
	}

	class BackLoad extends Thread {
		int book_no;
		int chapter_no;
		boolean msgbox=true;
		
		public BackLoad(int _book_no, int _chapter_number) {
			book_no = _book_no;
			chapter_no = _chapter_number;
		}
		
		public BackLoad(int _book_no, int _chapter_number,boolean _msgbox) {
			book_no = _book_no;
			chapter_no = _chapter_number;
			msgbox=_msgbox;
		}

		public void run() {
			try {
				if(msgbox)
				{
				loading.setMessage("Loading.. "
						+ StrongDB.getBookName(book_no - 1) + " - "
						+ chapter_no);
				loading.setVisible(true);
				}
				StrongDB.getChapterPanel(chapterPanel, book_no, chapter_no);
				chapter_jsp.doLayout();
				Point p = frame.getLocation();
				Rectangle r = frame.getBounds();
				frame.pack();
				frame.setLocation(p);
				frame.setBounds(r);
				chapterPanel.scrollRectToVisible(new Rectangle(0, 0, 10, 10));
				if(msgbox)
					loading.setVisible(false);
			} catch (Exception e1) {
				JOptionPane.showMessageDialog(frame, "Error: "
						+ e1.getLocalizedMessage());
				e1.printStackTrace();
			}
		}
	}

}
